@@ -1,14 +1,15 @@
-from django.conf.urls.defaults import patterns, include, url
+from django.conf.urls import include, url
 from django.contrib import admin
+
 admin.autodiscover()
 
-urlpatterns = patterns('',
+urlpatterns = [
     url(r'^admin/apps/tinymce/', include('tinymce.urls')),
-    url(r'^admin/apps/tags/', include('taggit_autocomplete_modified.urls')),
-    url(r'^admin/', include(admin.site.urls)),
+    url(r'^admin/apps/tags/', include('taggit_autosuggest.urls')),
+    url(r'^admin/', admin.site.urls),
 
     url(r'^comments/', include('fluent_comments.urls')),
     #url(r'^forms/', include('form_designer.urls')),
 
     url(r'', include('fluent_blogs.urls')),
-)
+]
