@@ -1,9 +1,11 @@
-from os.path import join, dirname
+from os.path import dirname, join
+
 from example_standalone.settings import *
 
 INSTALLED_APPS += (
     # Add fluent pages with a simple page type:
     'fluent_pages',
+    #'fluent_pages.pagetypes.fluentpage',
     'fluent_pages.pagetypes.flatpage',
 
     # Add the page type for adding the "blogs" root to the page tree.
@@ -17,6 +19,6 @@ INSTALLED_APPS += (
 
 ROOT_URLCONF = 'example_fluent_pages.urls'
 
-TEMPLATE_DIRS = (
+TEMPLATES[0]['DIRS'] = (
     join(dirname(__file__), "templates"),
 )
