@@ -1,11 +1,12 @@
-from django.conf.urls.defaults import patterns, include, url
+from django.conf.urls import include, url
 from django.contrib import admin
+
 admin.autodiscover()
 
-urlpatterns = patterns('',
+urlpatterns = [
     url(r'^admin/apps/tinymce/', include('tinymce.urls')),
-    url(r'^admin/apps/tags/', include('taggit_autocomplete_modified.urls')),
-    url(r'^admin/', include(admin.site.urls)),
+    url(r'^admin/apps/tags/', include('taggit_autosuggest.urls')),
+    url(r'^admin/', admin.site.urls),
 
     url(r'^comments/', include('fluent_comments.urls')),
     #url(r'^forms/', include('form_designer.urls')),
@@ -14,4 +15,4 @@ urlpatterns = patterns('',
     # Instead, create a "blogpage" in the page tree.
     # all URLs will be displayed there.
     url(r'', include('fluent_pages.urls')),
-)
+]
