@@ -1,12 +1,24 @@
 """
 A query interface to retrieve blog models and tags.
 """
-from calendar import monthrange, isleap
+import sys
+from calendar import monthrange
 from datetime import datetime, timedelta
+
+import django
+from django.conf import settings
+from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
 from django.db.models.aggregates import Count
-from fluent_blogs.models.db import get_entry_model
-from fluent_blogs.utils.compat import utc
+from django.utils.timezone import utc
+from parler.models import TranslatableModel
+
+from fluent_blogs import appsettings
+from fluent_blogs.models.db import get_category_model, get_entry_model
+
+if sys.version_info[0] >= 3:
+    basestring = str
+
 
 __all__ = (
     'query_entries',
@@ -26,6 +38,11 @@ ENTRY_ORDER_BY_FIELDS = {
     'year': ('publication_date',),
 }
 
+if django.VERSION >= (1, 11):
+    # Django 1.10 doesn't support early importing.
+    User = get_user_model()
+    ENTRY_ORDER_BY_FIELDS['author_slug'] = 'author__{}'.format(User.USERNAME_FIELD)
+
 TAG_ORDER_BY_FIELDS = {
     'slug': ('slug',),
     'name': ('name',),
@@ -50,7 +67,7 @@ def _get_order_by(order, orderby, order_by_fields):
 
     # Default to descending for some fields, otherwise be ascending
     is_desc = (not order and orderby in ORDER_BY_DESC) \
-           or (order or 'asc').lower() in ('desc', 'descending')
+        or (order or 'asc').lower() in ('desc', 'descending')
 
     if is_desc:
         return map(lambda name: '-' + name, db_fieldnames)
@@ -58,16 +75,17 @@ def _get_order_by(order, orderby, order_by_fields):
         return db_fieldnames
 
 
-def query_entries(queryset=None,
-        year=None, month=None, day=None,
-        category=None, category_slug=None,
-        tag=None, tag_slug=None,
-        author=None, author_slug=None,
-        future=False,
-        order=None,
-        orderby=None,
-        limit=None,
-    ):
+def query_entries(
+    queryset=None,
+    year=None, month=None, day=None,
+    category=None, category_slug=None,
+    tag=None, tag_slug=None,
+    author=None, author_slug=None,
+    future=False,
+    order=None,
+    orderby=None,
+    limit=None,
+):
     """
     Query the entries using a set of predefined filters.
     This interface is mainly used by the ``get_entries`` template tag.
@@ -75,6 +93,9 @@ def query_entries(queryset=None,
     if queryset is None:
         queryset = get_entry_model().objects.all()
 
+    if appsettings.FLUENT_BLOGS_FILTER_SITE_ID:
+        queryset = queryset.parent_site(settings.SITE_ID)
+
     if not future:
         queryset = queryset.published()
 
@@ -88,34 +109,33 @@ def query_entries(queryset=None,
     # The main category/tag/author filters
     if category:
         if isinstance(category, basestring):
-            queryset = queryset.filter(categories__slug=category)
+            queryset = queryset.categories(category)
         elif isinstance(category, (int, long)):
             queryset = queryset.filter(categories=category)
         else:
             raise ValueError("Expected slug or ID for the 'category' parameter")
     if category_slug:
-        queryset = queryset.filter(categories__slug=category)
+        queryset = queryset.categories(category)
 
     if tag:
         if isinstance(tag, basestring):
-            queryset = queryset.filter(tags__slug=tag)
+            queryset = queryset.tagged(tag)
         elif isinstance(tag, (int, long)):
             queryset = queryset.filter(tags=tag)
         else:
             raise ValueError("Expected slug or ID for 'tag' parameter.")
     if tag_slug:
-        queryset = queryset.filter(tags__slug=tag)
+        queryset = queryset.tagged(tag)
 
     if author:
         if isinstance(author, basestring):
-            queryset = queryset.filter(author__username=author)
+            queryset = queryset.authors(author)
         elif isinstance(author, (int, long)):
             queryset = queryset.filter(author=author)
         else:
             raise ValueError("Expected slug or ID for 'author' parameter.")
     if author_slug:
-        queryset = queryset.filter(author__username=author_slug)
-
+        queryset = queryset.authors(author_slug)
 
     # Ordering
     if orderby:
@@ -136,13 +156,26 @@ def query_tags(order=None, orderby=None, limit=None):
     This interface is mainly used by the ``get_tags`` template tag.
     """
     from taggit.models import Tag, TaggedItem    # feature is still optional
+
+    # Get queryset filters for published entries
     EntryModel = get_entry_model()
     ct = ContentType.objects.get_for_model(EntryModel)  # take advantage of local caching.
-    entry_tag_ids = TaggedItem.objects.filter(content_type=ct).values_list('tag_id')
+
+    entry_filter = {
+        'status': EntryModel.PUBLISHED
+    }
+    if appsettings.FLUENT_BLOGS_FILTER_SITE_ID:
+        entry_filter['parent_site'] = settings.SITE_ID
+
+    entry_qs = EntryModel.objects.filter(**entry_filter).values_list('pk')
 
     # get tags
-    queryset = Tag.objects.filter(id__in=entry_tag_ids)
-    queryset = queryset.annotate(count=Count('taggit_taggeditem_items'))  # Related name differed in Django 1.2
+    queryset = Tag.objects.filter(
+        taggit_taggeditem_items__content_type=ct,
+        taggit_taggeditem_items__object_id__in=entry_qs
+    ).annotate(
+        count=Count('taggit_taggeditem_items')
+    )
 
     # Ordering
     if orderby:
@@ -157,6 +190,17 @@ def query_tags(order=None, orderby=None, limit=None):
     return queryset
 
 
+def get_category_for_slug(slug, language_code=None):
+    """
+    Find the category for a given slug
+    """
+    Category = get_category_model()
+    if issubclass(Category, TranslatableModel):
+        return Category.objects.active_translations(language_code, slug=slug).get()
+    else:
+        return Category.objects.get(slug=slug)
+
+
 def get_date_range(year=None, month=None, day=None):
     """
     Return a start..end range to query for a specific month, day or year.
